//
//  PlaylistViewController.swift
//  ColorfulWallpaper
//
//  Created by ni on 2024/9/18.
//

import Combine
import Foundation
import TSVideoKit

class PlaylistViewController: LWBGViewController, UITableViewDelegate {
    var viewModel: PlayListViewModel = .init()
    weak var importMenuView: BubbleMenuView?

    lazy var searchBar: MusicSearchBar = {
        let bar = MusicSearchBar()
        bar.addTarget(self, action: #selector(showSearchViewController), for: .touchUpInside)
        return bar
    }()

    lazy var hStackView: UIStackView = {
        let stack: UIStackView = .hStack
        stack.spacing = 16
        stack.alignment = .center
        return stack
    }()

    lazy var importButton: UIButton = {
        let bt = UIButton(frame: CGRect(x: 0, y: 0, width: 40, height: 40))
        bt.backgroundColor = UIColor.white.withAlphaComponent(0.1)
        bt.setImage(.icNavImport, for: .normal)
        bt.customCornerRadius = 20
        bt.backgroundColor = .white
        return bt
    }()

    lazy var topViewHStack: UIStackView = {
        let mStack: UIStackView = .hStack
        mStack.spacing = 16
        mStack.distribution = .fillEqually
        return mStack
    }()

//    lazy var likeItem: PlayListTopItemView = {
//        let item = PlayListTopItemView()
//        item.bgImageView.image = .icTopLike
//        item.titleLabel.text = "Like".localized()
//        item.countLabel.text = "0"
//        item.addTarget(self, action: #selector(showLikeViewController), for: .touchUpInside)
//        return item
//    }()
//
//    lazy var recentlyItem: PlayListTopItemView = {
//        let item = PlayListTopItemView()
//        item.bgImageView.image = .icTopRecently
//        item.titleLabel.text = "Recently".localized()
//        item.countLabel.text = "0"
//        item.addTarget(self, action: #selector(showRecentlyList), for: .touchUpInside)
//        return item
//    }()

    lazy var listTitle: UILabel = .simpleLabel(text: "My Playlist".localized(), font: .boldSystemFont18, color: .black)

    lazy var listView: UITableView = {
        let tabView = UITableView()
        tabView.dataSource = viewModel
        tabView.delegate = self
        tabView.separatorStyle = .none
        tabView.register(PlayListCell
            .self, forCellReuseIdentifier: "PlayListCell")
        tabView.register(PlayListAddCell.self, forCellReuseIdentifier: "PlayListAddCell")
        tabView.register(PlayListFavouriteCell.self, forCellReuseIdentifier: "PlayListFavouriteCell")
        tabView.backgroundColor = .clear
        tabView.contentInset = UIEdgeInsets(top: 0, left: 0, bottom: 80, right: 0)
        return tabView
    }()

    private var canncellable: [AnyCancellable] = []

    override func viewDidLoad() {
        super.viewDidLoad()
        bgImageView.image = .viewMainBg
        navigationController?.navigationBar.isHidden = true
        addNotifaction()
        addTargets()
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        PlayerManager.shared.miniBar.isHidden = false
        PlayerManager.shared.rootVc?.moveUpMiniBar()
        viewModel.requestData()
    }


    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
    }

    override func addNotifaction() {
        super.addNotifaction()

        viewModel.$playLists.receive(on: DispatchQueue.main).sink { _ in
            self.reloadListData()
        }.store(in: &canncellable)

        NotificationCenter.default.addObserver(self, selector: #selector(reloadData), name: .K_RefreshNotifaction, object: nil)
    }

    @objc func reloadData() {
        viewModel.requestData()
    }

    func reloadListData() {
        updateCount()
        listView.reloadData()
    }

    func newPlaylist() {
        let alertVC = UIAlertController(title: nil, message: "Playlist Name".localized(), preferredStyle: .alert)
        alertVC.addTextField { textField in
            textField.placeholder = "input name".localized()
            textField.font = UIFont.systemFont(ofSize: 16)
        }
        let ok = UIAlertAction(title: "OK".localized(), style: .default) { [weak self] _ in
            if let name = alertVC.textFields?.first?.text, !name.isEmpty {
                self?.viewModel.creatPlayList(name: name) { listid in
                    if let id = listid {
                        self?.didFinishedCreatedPlaylist(id: id)
                    }
                }
            }
        }
        let cancel = UIAlertAction(title: "Cancel".localized(), style: .cancel)
        alertVC.addAction(cancel)
        alertVC.addAction(ok)
        present(alertVC, animated: true)
    }

    func didFinishedCreatedPlaylist(id: UUID) {
    }

    override func addChildren() {
        super.addChildren()
        view.addSubview(hStackView)
        hStackView.addArrangedSubview(searchBar)
        hStackView.addArrangedSubview(importButton)
        view.addSubview(topViewHStack)
//        topViewHStack.addArrangedSubview(likeItem)
//        topViewHStack.addArrangedSubview(recentlyItem)
        view.addSubview(listTitle)
        view.addSubview(listView)
    }

    func updateCount() {
//        likeItem.countLabel.text = "\(viewModel.likeCount)"
//        recentlyItem.countLabel.text = "\(viewModel.recentlyCount)"
        let attr1 = NSMutableAttributedString(string: "My Playlist", attributes: [.font: UIFont.boldSystemFont18, .foregroundColor: UIColor.black])
        let attr2 = NSAttributedString(string: " (\(viewModel.playLists.count))", attributes: [.font: UIFont.boldSystemFont18, .foregroundColor: UIColor.hexColor("#999999")])
        attr1.append(attr2)
        listTitle.attributedText = attr1
    }

    override func makeConstarints() {
        super.makeConstarints()

        hStackView.snp.makeConstraints { make in
            make.leading.trailing.equalToSuperview().inset(16)
            make.top.equalTo(view.safeAreaLayoutGuide.snp.top)
            make.height.equalTo(44)
        }
        
        searchBar.snp.makeConstraints { make in
            make.height.equalTo(40)
        }

        importButton.snp.makeConstraints { make in
            make.width.height.equalTo(40)
        }

        topViewHStack.snp.makeConstraints { make in
            make.top.equalTo(hStackView.snp.bottom).offset(20)
            make.leading.trailing.equalToSuperview().inset(16)
            make.height.equalTo(100)
        }

        listTitle.snp.makeConstraints { make in
            make.top.equalTo(topViewHStack.snp.bottom).offset(12)
            make.horizontalEdges.equalToSuperview().inset(16)
            make.height.equalTo(48)
        }

        listView.snp.makeConstraints { make in
            make.top.equalTo(listTitle.snp.bottom)
            make.bottom.leading.trailing.equalToSuperview()
        }
    }

    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if indexPath.section == 0 {
            newPlaylist()
        } else {
//            ADManager.shared.showAd(scene: ADScene.playlist, from: self) { state in
//                if state == .finished || state == .fail {
//                    if indexPath.row < self.viewModel.playLists.count {
            let list = viewModel.playLists[indexPath.row]
            let vModel: PlayListDetaiViewModel = .init()
            vModel.playlist = list
            vModel.listType = .playlist
            let vc = PlaylistDetailViewController(viewModel: vModel)
            vc.hidesBottomBarWhenPushed = true
            navigationController?.pushViewController(vc, animated: true)
//                    }
//                }
//            }
        }
    }

    deinit {
        NotificationCenter.default.removeObserver(self)
        canncellable.removeAll()
    }
}
