//
//  MusicEmptyView.swift
//  PhysicalWallPaper
//
//  Created by nkl on 2024/11/13.
//

//import ADManager
import Foundation
import TSVideoKit

extension SongListViewController {
    func addTargets() {
        searchBar.addTarget(self, action: #selector(showSearchViewController), for: .touchUpInside)
        importButton.addTarget(self, action: #selector(showImportMenuView), for: .touchUpInside)
    }

    @objc func showSearchViewController() {
        let vc = LocalSearchViewController()
        vc.hidesBottomBarWhenPushed = true
        self.navigationController?.pushViewController(vc, animated: true)
        
//        if PurchaseManager.default.isVip {
//            if TSConfiguration.isYs {
//                let vc = SearchOnlineViewController()
//                vc.hidesBottomBarWhenPushed = true
//                navigationController?.pushViewController(vc, animated: true)
//            } else {
//                let vc = LocalSearchViewController()
//                vc.hidesBottomBarWhenPushed = true
//                navigationController?.pushViewController(vc, animated: true)
//            }
//        } else {
//            ADManager.shared.showAd(scene: ADScene.searchInsert, from: self) { state in
//                if state == .finished || state == .fail {
//                    if TSConfiguration.isYs {
//                        ADManager.shared.prepareAd(scenes: [ADScene.downloadInsert])
//                        let vc = SearchOnlineViewController()
//                        vc.hidesBottomBarWhenPushed = true
//                        self.navigationController?.pushViewController(vc, animated: true)
//                    } else {
//                        let vc = LocalSearchViewController()
//                        vc.hidesBottomBarWhenPushed = true
//                        self.navigationController?.pushViewController(vc, animated: true)
//                    }
//                }
//            }
//        }
    }

    @objc func showImportMenuView() {
        let menuView = BubbleMenuView()
        menuView.position = FitManager.isAr ? .topRight : .topLeft
        menuView.actionHanlder = { type in
            self.menuClick(type: type)
        }
        view.addSubview(menuView)
        menuView.snp.makeConstraints { make in
            make.leading.equalTo(self.importButton).offset(-16)
            make.trailing.equalToSuperview().offset(-16)
            make.top.equalTo(self.importButton.snp.bottom).offset(0)
        }
        importMenuView = menuView
    }

    func menuClick(type: ImportSource) {
        importMenuView?.removeFromSuperview()
        if type == .library {
            ImportFilesManager.shared.openPhotoLibrary(parent: self)
        } else if type == .file {
            ImportFilesManager.shared.openFileDocument(parent: self, completion: nil)
        }
    }

    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        if indexPath.section == 0 {
            guard let cell = cell as? SongDownloadCell, indexPath.row < viewModel.unFinishedVideo.count else {
                return
            }

            let viewModel = viewModel.unFinishedVideo[indexPath.row]

            cell.bindViewModel(viewModel: viewModel, isOnline: false)

            cell.downloadStateChangeBlock = { [weak self] state in
                guard let self = self else {
                    return
                }
                switch state {
                case .idle(isAnimate: false), .idle(isAnimate: true):
//                    if PurchaseManager.default.isVip {
                    viewModel.startDownload()
//                    if !PurchaseManager.default.isVip {
//                        ADManager.shared.showAd(scene: ADScene.downloadInsert, from: self)
//                    }
//                    } else {
//                        let loading = ADLoadingViewController()
//                        loading.adScene = .downloadReward
//                        loading.finishedHandler = { finished in
//                            if finished {
//                                viewModel.startDownload()
//                            }
//                        }
//                        present(loading, animated: false)
//                    }
                case .pause:
                    viewModel.resumeDownload()
                    break
                case .done:
                    if let videoModel = viewModel.dataVideo {
//                        self.viewModel.showOperationViewController(video: videoModel)
                    }
                    break
                case .downloading:
                    viewModel.pauseDownload()
                    break
                case .retry:
                    /// todo.kailen
                    viewModel.retryDownload()
                    break
                case .loading:
                    break
                }
            }
        }
    }
}
